@echo off 


rem last update was on February 16, 2014

rem Batch file to perform several tasks used to prepare vanilla textures for optimization using DDSopt
rem as per steps in the DDSopt guide (http://wiki.step-project.com/Guide:DDSopt#tab=DDSopt_Optimization).
rem It is only useful when being used with other steps described in this guide.
rem If this batch file is edited, this MUST BE DONE with an editor that does not add additional characters (e.g., line feed, carriage return)
rem these characters are often invisible but can sometimes cause the batch file to malfunction
rem DO NOT USE a default Windows text editor to edit this file
rem Editors like Notepad++ (which was used to create this file), TextPad, UltraEdit are fine
 
rem It expects that there is a Working directory containing the "Vanilla Extracted" directory
rem This directory should not be in any User Access Controlled (UAC) directory such as Program Files or Program Files (x86).
rem It expects that the BSA contents have already been extracted into folders in the  "Vanilla Extracted" directory
rem It expects a folder in "Vanilla Extracted" labeled "STD"
rem If the HRDLC are being optimized, the extracted BSA contents should be in "HRDLC1", "HRDLC2", "HRDLC3" in "Vanilla Extracted"
rem If there are any optional DLC being used their BSAs should have also been extracted into "Vanilla Extracted"
rem with folder names of "Dawnguard", "Hearthfires", and "Dragonborn" (based on the DLC being used).
rem
rem The batch file first copies .png files in the STD folder in "Vanilla Extracted" to the STD folder in "Vanilla Optimized"
rem The .png files in the DLC BSAsremain there so they don't need to be copied
rem The batch file then copies 10 compressed, smooth normal maps to Vanilla Optimized (added 9/12/2013)
rem  to avoid optimizing them with DDSopt them as the optimization exarbates some vanilla artifacts
rem
rem To prepare the vanilla textures this batch file copies vanilla texture files (files ending in .dds) from folders 
rem in "Vanilla Extracted" to "Vanilla Ordinary Textures" while excluding normal map textures (file names ending in _n.dds and _msn.dds)
rem from the HRDLC
rem Any large compressed normal maps (2Kx2K) in the Skyrim - Textures.bsa and the addon DLC BSA are copied to "Vanilla Normal Maps"
rem The rest of the compressed normal map textures in Skyrim - Textures.bsa and the addon DLC BSA are all low to medium resolution
rem Since their resolutions are not very high, these are optimized with the rest of the textures without changing resolution
rem
rem The batch file copies uncompressed normal map textures from the HRDLC and uncompressed textures in Skyrim - textures 
rem in "Vanilla Extracted" to the corresponding folders in the "Vanilla Uncompressed Textures" directory.
rem
rem Finally, the batch file copies the rest of the normal map textures from the HRDLC in "Vanilla Extracted"
rem and any large normal maps in the other directories to the corresponding folders (as mentioned above) in the "Vanilla Normal Maps" directory.
rem
rem While the batch file runs there will be a command window that echoes the commands
rem This allows monitoring the progress of the batch processing if desired, although it is not required.
 
rem the initial 2 lines should ensure that the system provides the correct working directory name in %cd%
rem This is needed for a small percentage of systems, probably because of interaction with Windows UAC on those systems

REM tests added 8/18/2013 to make sure the batch files are in the right folder and the folder structures are correct
rem 10/12/2013 "Vanilla Uncompressed Normal Maps" folder renamed "Vanilla Uncompressed Textures", and
rem uncompressed vanilla tint\tone masks are now copied into the "Vanilla Uncompressed Textures" folder vs. the "Vanilla Ordinary Textures" folder
rem 2/4/2014 test added for presence of windows\system32\robocopy.exe
rem 2/4/2014 all MSN files go to "Vanilla Optimized" so they are not corrupted
rem 2/16/12 removed "HRDLC3" folder from "Vanilla Uncompressed Textures"; these uncompressed MSN textures now go directly to "Vanilla Optimized"



IF NOT EXIST "%Windir%\system32\robocopy.exe" echo ERROR: The Robocopy system program isn't available; fix this before proceeding

@setlocal enableextensions
@cd /d "%~dp0"
 
echo.
echo.
echo The system says the Working Directory is   %cd%
echo This directory should include the "Vanilla Extracted" directory
echo If any of the above is incorrect terminate the processing and fix the problem
echo.

rem test added 12/19/2013 and modified 1/13/2014 to test for administrative privileges
rem taken from http://stackoverflow.com/questions/4051883/batch-script-how-to-check-for-admin-rights
rem administrative privilege isn't usually needed
:testPrivilegies
    set guid=%random%%random%-%random%-%random%-%random%-%random%%random%%random%
    mkdir %WINDIR%\%guid%>nul 2>&1
    rmdir %WINDIR%\%guid%>nul 2>&1
    IF NOT %ERRORLEVEL%==0 echo  Administrative privileges are not enabled; this is usually not a problem
        


IF NOT EXIST "%cd%\Vanilla Extracted" echo  Error: The Vanilla Extracted directory is not in the working directory; fix this before proceeding
echo.
echo.
echo  ---------------------
echo On some systems the batch file may take a long time (30 minutes or so) and might not seem to be working
echo If it starts successfully and gets through a step or two it is almost certainly working so be patient with it
echo it's doing the best it can
echo ---------------------
echo Hit enter to continue after the pause
pause


 
IF NOT EXIST "%cd%\Vanilla Ordinary Textures\NUL"  MD "Vanilla Ordinary Textures"
IF NOT EXIST "%cd%\Vanilla Normal Maps\NUL"  MD "Vanilla Normal Maps"
IF NOT EXIST "%cd%\Vanilla Optimized\NUL"  MD "Vanilla Optimized"
IF NOT EXIST "%cd%\Vanilla Uncompressed Textures\NUL"  MD "Vanilla Uncompressed Textures"
cd "Vanilla Optimized"
IF NOT EXIST \STD\NUL  MD STD
cd ..

cd "Vanilla Ordinary Textures"
IF EXIST "..\Vanilla Extracted\HRDLC1" IF NOT EXIST \HRDLC1\NUL  MD HRDLC1
IF EXIST "..\Vanilla Extracted\HRDLC2" IF NOT EXIST \HRDLC2\NUL  MD HRDLC2
IF EXIST "..\Vanilla Extracted\HRDLC3" IF NOT EXIST \HRDLC3\NUL  MD HRDLC3
cd ..

cd "Vanilla Uncompressed Textures"
IF EXIST "..\Vanilla Extracted\HRDLC1" IF NOT EXIST \HRDLC1\NUL  MD HRDLC1
IF EXIST "..\Vanilla Extracted\HRDLC2" IF NOT EXIST \HRDLC2\textures\terrain\NUL  MD HRDLC2\textures\terrain\
rem HRDLC3 uncompressed textures are now put directly in "Vanilla Optimized"
rem IF EXIST "..\Vanilla Extracted\HRDLC3" IF NOT EXIST \HRDLC3\NUL  MD HRDLC3
IF EXIST "..\Vanilla Extracted\STD" IF NOT EXIST "\STD\textures\actors\character\character assets\tintmasks\NUL"  MD "STD\textures\actors\character\character assets\tintmasks\"
cd .. 


REM Get the starting time
set STARTTIME=%TIME%


@echo on

rem copy the PNG files from Skyrim - textures to the Vanilla Optimized directory
IF EXIST "%cd%\Vanilla Extracted\STD" %Windir%\system32\robocopy "%cd%\Vanilla Extracted\STD" "%cd%\Vanilla Optimized\STD" *.png /s /nfl /ndl /njh /njs /ns /nc /np >>log.txt

rem copy some compressed, smooth normal maps to Vanilla Optimized to avoid optimizing them as the optimization exarbates some  vanilla artifacts
IF EXIST "%cd%\Vanilla Extracted\STD" %Windir%\system32\robocopy "%cd%\Vanilla Extracted\STD" "%cd%\Vanilla Optimized\STD" coastbeach0?_n.dds dirt0?_n.dds dirtpath0?_n.dds dirtsnowpath0?_n.dds snow0?_n.dds tundra0?_n.dds /s /nfl /ndl /njh /njs /ns /nc /np >>log.txt

rem Copy texture files (from folders other than the HRDLC) to "Vanilla Ordinary Textures", excluding any tangent space normal map textures
%Windir%\system32\robocopy "%cd%\Vanilla Extracted" "%cd%\Vanilla Ordinary Textures" *.dds /s  /xd HRDLC1 HRDLC2 HRDLC3 /xf  *_n.dds /nfl /ndl /njh /njs /ns /nc /np >>log.txt
rem Copy small and medium size tangent space normal map textures from folders other than the 3 HRDLC to "Vanilla Ordinary Textures"
rem excluding the compressed smooth vanilla normal maps copied previously
%Windir%\system32\robocopy "%cd%\Vanilla Extracted" "%cd%\Vanilla Ordinary Textures" *_n.dds /s /max:2999999 /xd HRDLC1 HRDLC2 HRDLC3 /xf coastbeach0?_n.dds dirt0?_n.dds dirtpath0?_n.dds dirtsnowpath0?_n.dds snow0?_n.dds tundra0?_n.dds /nfl /ndl /njh /njs /ns /nc /np >>log.txt
rem Copy any large tangent space normal map textures (from folders other than the HRDLC) to "Vanilla Normal Maps"
rem  This step copies 5 large tangent space normal map files from STD and 5 large tangent space normal map files from Dawnguard if present
%Windir%\system32\robocopy "%cd%\Vanilla Extracted" "%cd%\Vanilla Normal Maps" *_n.dds /s /min:3000000 /xd HRDLC1 HRDLC2 HRDLC3 /nfl /ndl /njh /njs /ns /nc /np >>log.txt


rem Now copy texture files from the HRDLC to "Vanilla Ordinary Textures" excluding any normal map textures
IF EXIST "%cd%\Vanilla Extracted\HRDLC1" %Windir%\system32\robocopy "%cd%\Vanilla Extracted\HRDLC1" "%cd%\Vanilla Ordinary Textures\HRDLC1" *.dds /s /xf *_n.dds *_msn.dds /nfl /ndl /njh /njs /ns /nc /np >>log.txt
IF EXIST "%cd%\Vanilla Extracted\HRDLC2"  %Windir%\system32\robocopy "%cd%\Vanilla Extracted\HRDLC2" "%cd%\Vanilla Ordinary Textures\HRDLC2" *.dds /s /xf *_n.dds *_msn.dds /nfl /ndl /njh /njs /ns /nc /np >>log.txt
IF EXIST "%cd%\Vanilla Extracted\HRDLC3" %Windir%\system32\robocopy "%cd%\Vanilla Extracted\HRDLC3" "%cd%\Vanilla Ordinary Textures\HRDLC3" *.dds /s /xf *_n.dds *_msn.dds /nfl /ndl /njh /njs /ns /nc /np >>log.txt

rem Now copy 1 2Kx2K compressed HRDLC1 MSN file to "Vanilla Uncompressed Textures"
IF EXIST "%cd%\Vanilla Extracted\HRDLC1" %Windir%\system32\robocopy "%cd%\Vanilla Extracted\HRDLC1" "%cd%\Vanilla Uncompressed Textures\HRDLC1" *_msn.dds /s /nfl /ndl /njh /njs /ns /nc /np >>log.txt
rem Copy uncompressed HRDLC3 MSN texture files to "Vanilla Optimized" to ensure they are unchanged
rem If resolution of the 2Kx2K MSN is reduced it will be corrupted (although optimized with R5G6B5 format will work fine)
IF EXIST "%cd%\Vanilla Extracted\HRDLC3" %Windir%\system32\robocopy "%cd%\Vanilla Extracted\HRDLC3" "%cd%\Vanilla Optimized\HRDLC3" *_msn.dds /s /nfl /ndl /njh /njs /ns /nc /np >>log.txt
rem Copy uncompressed HRDLC2 terrain TSN texture files to "Vanilla Uncompressed Textures"
IF EXIST "%cd%\Vanilla Extracted\HRDLC2"  %Windir%\system32\robocopy "%cd%\Vanilla Extracted\HRDLC2\textures\terrain" "%cd%\Vanilla Uncompressed Textures\HRDLC2\textures\terrain" *_n.dds /s /nfl /ndl /njh /njs /ns /nc /np >>log.txt

rem copy uncompressed tint/tone maps from STD to "Vanilla Uncompressed Textures"
IF EXIST "%cd%\Vanilla Extracted\STD" %Windir%\system32\robocopy "%cd%\Vanilla Extracted\STD\textures\actors\character\character assets\tintmasks" "%cd%\Vanilla Uncompressed Textures\STD\textures\actors\character\character assets\tintmasks" *.dds /s /nfl /ndl /njh /njs /ns /nc /np >>log.txt
rem then remove these uncompressed files from "Vanilla Ordinary Textures"
IF EXIST "%cd%\Vanilla Uncompressed Textures\STD\textures\actors\character\character assets\tintmasks"  rd "%cd%\Vanilla Ordinary Textures\STD\textures\actors\character\character assets\tintmasks" /s /q >>log.txt

rem Now copy the rest of the HRDLC tangent space normal maps (the compressed normal maps) to "Vanilla Normal Maps"

IF EXIST "%cd%\Vanilla Extracted\HRDLC1" %Windir%\system32\robocopy "%cd%\Vanilla Extracted\HRDLC1" "%cd%\Vanilla Normal Maps\HRDLC1" *_n.dds /s /nfl /ndl /njh /njs /ns /nc /np >>log.txt
IF EXIST "%cd%\Vanilla Extracted\HRDLC2"  %Windir%\system32\robocopy "%cd%\Vanilla Extracted\HRDLC2" "%cd%\Vanilla Normal Maps\HRDLC2" *_n.dds /s /nfl /ndl /njh /njs /ns /nc /np >>log.txt
IF EXIST "%cd%\Vanilla Extracted\HRDLC3" %Windir%\system32\robocopy "%cd%\Vanilla Extracted\HRDLC3" "%cd%\Vanilla Normal Maps\HRDLC3" *_n.dds /s /nfl /ndl /njh /njs /ns /nc /np >>log.txt
rem Now remove the terrain subfolder from "Vanilla Normal Maps\HRDLC2\textures\"; this folder has only uncompressed normal maps
IF EXIST "%cd%\Vanilla Normal Maps\HRDLC2\textures\terrain"  rd "%cd%\Vanilla Normal Maps\HRDLC2\textures\terrain" /s /q >>log.txt


@echo off


rem get time of completion
set ENDTIME=%TIME%

rem from http://stackoverflow.com/questions/9922498/calculate-time-difference-in-batch-file

rem Change formatting for the start and end times
for /F "tokens=1-4 delims=:.," %%a in ("%STARTTIME%") do (
    set /A "start=(((%%a*60)+1%%b %% 100)*60+1%%c %% 100)*100+1%%d %% 100"
)

for /F "tokens=1-4 delims=:.," %%a in ("%ENDTIME%") do (
    set /A "end=(((%%a*60)+1%%b %% 100)*60+1%%c %% 100)*100+1%%d %% 100"
)

rem Calculate the elapsed time by subtracting values
set /A elapsed=end-start

rem Format the results for output
set /A hh=elapsed/(60*60*100), rest=elapsed%%(60*60*100), mm=rest/(60*100), rest%%=60*100, ss=rest/100, cc=rest%%100
if %hh% lss 10 set hh=0%hh%
if %mm% lss 10 set mm=0%mm%
if %ss% lss 10 set ss=0%ss%
if %cc% lss 10 set cc=0%cc%

set DURATION=%hh%:%mm%:%ss%,%cc%


echo.
echo.
echo Start    : %STARTTIME%
echo Finish   : %ENDTIME%
echo          ---------------
echo Duration : %DURATION%

echo.
echo The batch file processing has finished.
echo  Check the log file (log.txt). The log file should not show any errors and should have 16 blank lines.
echo.
pause

